{% extends "admin/base_site.html" %}
{% load static %}
{% block content %}
  <form method="post">{% csrf_token %}
      {{ form.as_p }}
      <input type="submit" value="Run Reports">
  </form>
{% endblock %}
