{% extends "admin/change_list.html" %}
{% load i18n %}
{% block object-tools-items %}
  <li>
    <a href="{% url 'admin:choose_report_definition' %}?{{ request.GET.urlencode }}" class="button">Generate Reports</a>
  </li>
  {{ block.super }}
{% endblock %}
